-- Soundness of the determinism checking.
module Soundness where

open import Overlaps hiding (initial; preserved)
open import Alphabet hiding (initial; preserved)
open import Relation.Binary.PropositionalEquality
open import Data.Empty
open import Typing
open import Calculus
open import Determinism

-- Progress.
initial : ∀ {p} → D p → Deterministic _/_ p
initial (then d₁ d₂ disj) (then₁ e₁) (then₁ e₂) =
  cong then₁ (initial d₁ e₁ e₂)
initial (then d₁ d₂ disj) (then₁ e₁) (then₂ φ e₂) =
  ⊥-elim (disj (Overlaps.initial e₁ (recover-φ φ)) e₂)
initial (then d₁ d₂ disj) (then₂ φ e₁) (then₁ e₂) =
  ⊥-elim (disj (Overlaps.initial e₂ (recover-φ φ)) e₁)
initial (then d₁ d₂ disj) (then₂ φ₁ e₁) (then₂ φ₂ e₂) =
  cong (then₂ φ₁) (initial d₂ e₁ e₂)
initial (d|| d₁ d₂ disj) (||₁ e₁) (||₁ e₂) =
  cong ||₁ (initial d₁ e₁ e₂)
initial (d|| d₁ d₂ disj) (||₁ e₁) (||₂ e₂) =
  ⊥-elim (disj (Alphabet.initial e₁) (Alphabet.initial e₂))
initial (d|| d₁ d₂ disj) (||₂ e₁) (||₁ e₂) =
  ⊥-elim (disj (Alphabet.initial e₂) (Alphabet.initial e₁))
initial (d|| d₁ d₂ disj) (||₂ e₁) (||₂ e₂) =
  cong ||₂ (initial d₂ e₁ e₂)
initial (d+ d₁ d₂ disj) (+₁ e₁) (+₁ e₂) =
  cong +₁ (initial d₁ e₁ e₂)
initial (d+ d₁ d₂ disj) (+₁ e₁) (+₂ e₂) =
  ⊥-elim (disj e₁ e₂)
initial (d+ d₁ d₂ disj) (+₂ e₁) (+₁ e₂) =
  ⊥-elim (disj e₂ e₁)
initial (d+ d₁ d₂ disj) (+₂ e₁) (+₂ e₂) =
  cong +₂ (initial d₂ e₁ e₂)
initial (d* d disj) (star e₁) (star e₂) =
  cong star (initial d e₁ e₂)
initial event event event = refl

-- Preservation.
preserved : ∀ {a p} → (e : p / a) → D p → D (next e)
preserved (then₁ e) (then d₁ d₂ disj) =
  then (preserved e d₁) d₂
    (λ o f → disj (Overlaps.preserved e o) f)
preserved (then₂ φ e) (then d₁ d₂ disj) =
  preserved e d₂
preserved (||₁ e) (d|| d₁ d₂ disj) =
  d|| (preserved e d₁) d₂
    (λ a₁ a₂ → disj (Alphabet.preserved e a₁) a₂)
preserved (||₂ e) (d|| d₁ d₂ disj) =
  d|| d₁ (preserved e d₂)
    (λ a₁ a₂ → disj a₁ (Alphabet.preserved e a₂))
preserved (+₁ e) (d+ d₁ d₂ disj) =
  preserved e d₁
preserved (+₂ e) (d+ d₁ d₂ disj) =
  preserved e d₂
preserved (star e) (d* d disj) =
  then (preserved e d) (d* d disj)
    (λ{o (star e) → disj o e})
preserved event event = ε
