-- Determinism checking.
module Typing where

open import Calculus
open import Overlaps
open import Alphabet
open import Data.Empty

-- What it means for two sets to be disjoint.
Disjoint : (A → Set) → (A → Set) → Set
Disjoint p q = ∀ {a} → p a → q a → ⊥

-- The typing rules, one for each kind of process.
data D : Process → Set where
  ε : D ε

  event : ∀ {a r} →
    D (event a r)

  then : ∀ {p q} →
    (d₁ : D p) →
    (d₂ : D q) →
    (disj : Disjoint (Overlaps p) (Consumes q)) →
    D (p # q)

  d|| : ∀ {p q} →
    (d₁ : D p) →
    (d₂ : D q) →
    (disj : Disjoint (Alphabet p) (Alphabet q)) →
    D (p || q)

  d+ : ∀ {p q} →
    (d₁ : D p) →
    (d₂ : D q) →
    (disj : Disjoint (Consumes p) (Consumes q)) →
    D (p + q)

  d* : ∀ {p} →
    (d : D p) →
    (∀ {a} → {e : p / a} → Disjoint (Overlaps (next e)) (Consumes p)) →
    D (p *)
